const etagCalculate = require('etag');
const logger = require('@/utils/logger');
const config = require('@/config').value;
const headers = {
    'Access-Control-Allow-Methods': 'GET',
    'Content-Type': 'application/xml; charset=utf-8',
    'Cache-Control': `public, max-age=${config.cache.routeExpire}`,
    'X-Content-Type-Options': 'nosniff',
};

module.exports = async (ctx, next) => {
    logger.info(`${ctx.url}, user IP: ${ctx.ips[0] || ctx.ip}`);
    ctx.set(headers);
    ctx.set({
        'Access-Control-Allow-Origin': config.allowOrigin || ctx.host,
    });

    await next();

    if (!ctx.body || ctx.response.get('ETag')) {
        ctx.status = 404;
        ctx.body = { code: 404, message: 'Not Found' };
        return;
    }

    const status = (ctx.status / 100) | 0;
    if (2 !== status) {
        return;
    }

    if (ctx.fresh) {
        ctx.status = 304;
        ctx.body = null;
    }
};
