const logger = require('@/utils/logger');
const config = require('@/config').value;

let Sentry;

if (config.sentry.dsn) {
    Sentry = Sentry || require('@sentry/node');
    Sentry.init({
        dsn: config.sentry.dsn,
    });
    Sentry.configureScope((scope) => {
        scope.setTag('node_name', config.nodeName);
    });

    logger.info('Sentry inited.');
}

module.exports = async (ctx, next) => {
    try {
        const time = +new Date();
        await next();
        if (config.sentry.dsn && +new Date() - time >= config.sentry.routeTimeout) {
            Sentry.withScope((scope) => {
                scope.setTag('route', ctx._matchedRoute);
                scope.setTag('name', ctx.request.path.split('/')[1]);
                scope.addEventProcessor((event) => Sentry.Handlers.parseRequest(event, ctx.request));
                Sentry.captureException(new Error('Route Timeout'));
            });
        }
    } catch (err) {
        let message = err;
        if (err.name && (err.name === 'HTTPError' || err.name === 'RequestError')) {
            message = err.message;

        } else if (err instanceof Error) {
            message = process.env.NODE_ENV === 'production' ? err.message : err.stack;
        }

        logger.error(`Error in ${ctx.request.path}: ${message}`);

        if (ctx.status === 403) {
            message = err.message;
        } else {
            ctx.status = 503;
            message = err.message;
        }

        ctx.body = {
            code: ctx.status,
            error: {
                message
            },
        };

        if (!ctx.debug.errorPaths[ctx.request.path]) {
            ctx.debug.errorPaths[ctx.request.path] = 0;
        }

        ctx.debug.errorPaths[ctx.request.path]++;

        if (!ctx.debug.errorRoutes[ctx._matchedRoute]) {
            ctx._matchedRoute && (ctx.debug.errorRoutes[ctx._matchedRoute] = 0);
        }

        ctx._matchedRoute && ctx.debug.errorRoutes[ctx._matchedRoute]++;

        if (!ctx.state.debuged) {
            if (!ctx.debug.routes[ctx._matchedRoute]) {
                ctx._matchedRoute && (ctx.debug.routes[ctx._matchedRoute] = 0);
            }
            ctx._matchedRoute && ctx.debug.routes[ctx._matchedRoute]++;

            if (ctx.response.get('X-Koa-Redis-Cache') || ctx.response.get('X-Koa-Memory-Cache')) {
                ctx.debug.hitCache++;
            }
        }

        if (config.sentry.dsn) {
            Sentry.withScope((scope) => {
                scope.setTag('route', ctx._matchedRoute);
                scope.setTag('name', ctx.request.path.split('/')[1]);
                scope.addEventProcessor((event) => Sentry.Handlers.parseRequest(event, ctx.request));
                Sentry.captureException(err);
            });
        }

    }
};
